# restream

%NPM: restream%

Regular expression detection implemented as a `Transform` steam; and regex-based buffer replacement stream to replace incoming data on-the-fly.

```
yarn add -E restream
```


## Table of Contents

%TOC%

## API

The package contains the default `restream` and `Replaceable` functions.

```js
import restream, { Replaceable } from 'restream'
```

```### restream => Transform
[
  ["regex", "RegExp"]
]
```

Create a `Transform` stream which will buffer incoming data and push regex results when matches can be made, i.e. when `regex.exec` returns non-null value. When the `g` flag is added to the regex, multiple matches will be detected.

```js
/** yarn examples/restream.js **/
import restream from 'restream'
import { createReadable, createWritable } from './lib'

(async () => {
  try {
    const rs = createReadable('test-string-{12345}-{67890}')

    const stream = restream(/{(\d+)}/g) // create a transform stream
    rs.pipe(stream)

    const { data, ws } = createWritable()
    stream.pipe(ws)

    ws.once('finish', () => {
      console.log(data)
    })
  } catch (err) {
    console.error(err)
  }
})()
```

```js
[ [ '{12345}',
    '12345',
    index: 12,
    input: 'test-string-{12345}-{67890}' ],
  [ '{67890}',
    '67890',
    index: 20,
    input: 'test-string-{12345}-{67890}' ] ]
```

### `Replaceable`

A `Replaceable` transform stream can be used to transform data according to a single or multiple rules.

#### `Rule` Type

`Replaceable` uses rules to determine how to transform data. Below is the description of the `Rule` type.

```table
[
  ["Property", "Type", "Description"],
  ["re", "`RegExp`", "A regular expression."],
  ["replacement (1)", "`string`", "Replacement as a string. Here and below, it will be passed to the `string.replace(re, replacement)` native JavaScript method."],
  ["replacement (2)", "`function`", "Replacement as a sync function. See [MDN][2] for more documentation on how the replacer function should be implemented."],
  ["replacement (3)", "`async function`", "An asynchronous function to get replacements. The stream won't push any data until the replacer's promise is resolved. Due to implementation details, the regex will have to be run against incoming chunks twice, therefore it might be not ideal for heavy-load applications with many matches."]
]
```

```#### constructor => Replaceable
[
  ["rule", "Rule|Rules[]"]
]
```

Create a `Transform` stream which will make data available when an incoming chunk has been updated according to the specified rule or rules.

Matches can be replaced using a string, function or async function. When multiple rules are passed as an array, the string will be replaced multiple times if the latter rules also modify the data.

```js
/** yarn examples/Replaceable.js */
import Catchment from 'catchment'
import { Replaceable } from 'restream'
import { createReadable } from './lib'

(async () => {
  try {
    const dateRule = {
      re: /%DATE%/g,
      replacement: new Date().toLocaleString(),
    }
    const emRule = {
      re: /__(.+?)__/g,
      replacement(match, p1) {
        return `<em>${p1}</em>`
      },
    }
    const authorRule = {
      re: /^%AUTHOR: (.+?)%$/mg,
      async replacement(match, p1) {
        await new Promise(r => setTimeout(r,100))
        return `Author: <strong>${p1}</strong>`
      },
    }
    const replaceable = new Replaceable([
      dateRule,
      emRule,
      authorRule,
    ])
    const rs = createReadable(`
Hello __Fred__, your username is __fred__ and you have __5__ stars.
%AUTHOR: John%
on __%DATE%__
`)
    rs.pipe(replaceable)

    const { promise } = new Catchment({
      rs: replaceable,
    })
    const res = await promise

    console.log(res)
  } catch (err) {
    console.error(err)
  }
})()
```

Output:

```fs
Hello <em>Fred</em>, your username is <em>fred</em> and you have <em>5</em> stars.
Author: <strong>John</strong>
on <em>2018-6-20 19:59:18</em>
```

```#### DEPRECATED_replaceStream => Transform
[
  ["rule", "Rule|Rule[]"]
]
```

Used to create a `Replaceable` stream. Deprecated in favour of the class constructor (see above).

```js
/** yarn examples/replace-stream.js */
import Catchment from 'catchment'
import { replaceStream } from 'restream'
import { createReadable } from './lib'

(async () => {
  try {
    const stream = replaceStream([{
      re: /{{ user }}/,
      replacement: 'fred',
    }, {
      re: /{{ name }}/g,
      replacement: 'Fred',
    }, {
      re: /{{ stars }}/,
      replacement: '5',
    }])

    const rs = createReadable('Hello {{ name }}, your username is {{ user }} and you have {{ stars }} stars')
    rs.pipe(stream)
    const { promise } = new Catchment({
      rs: stream,
    })
    const res = await promise
    console.log(res)
  } catch (err) {
    console.error(err)
  }
})()
```

Output:

```fs
Hello Fred, your username is fred and you have 5 stars
```

<!-- , which can
be either a single regex object (`{ re: /regex/, replacement: 'hello-world' }`),
or an array of such objects. A `replacement` can be either a string, or a function,
which will be called by `str.replace` -->

---

(c) [Art Deco Code][1] 2018

[1]: https://artdeco.bz
[2]: https://developer.mozilla.org/en/docs/Web/JavaScript/Reference/Global_Objects/String/replace#Specifying_a_function_as_a_parameter
